/*
    PubTrans - Scheduling public transportation on PalmPilot handheld

    Copyright (C) 1999 Andreas Kretzer

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    e-mail: andi@kretzer-berlin.de
*/

#include <Common.h>
#include <System/SysAll.h>
#include <UI/UIAll.h>
#include "pubtrans.h"

#define TMPBUFSIZ	512

char *tmpstr(int len)
{
   static char buf[TMPBUFSIZ], *bufp = buf;
   static int i = 0;
   int allocsize = 16;	// minimum allocation unit
   char *ret;

   if (len > (TMPBUFSIZ >> 1)) return NULL;

   while (allocsize < len) allocsize *= 2;  // allow blocks of 16, 32, 64, ...
   if (len < allocsize) len = allocsize;

   if (len > (TMPBUFSIZ >> 1)) return NULL; // check again

   if ((i + len) > TMPBUFSIZ) i = 0;
   ret = bufp + i;
   i += len;

   return ret;
}

static char *cmp[32] = {
   NULL,
   " Str.", "-Str.", "str.",
   " Damm", "-Damm", "damm",
   " Platz", "-Platz", "platz",
   " Allee", "-Allee", "allee",
   " Weg", "-Weg", "weg",
   "Heide", "heide",
   "heimer", "heim",
   "bader", "bad",
   "dorfer", "dorf",
   "(S+U)", "(S)", "(U)",
   " Hafen", "-Hafen", "hafen",
   NULL, NULL
};

char * uncompress_string (unsigned char *s)
{
   char *ret;
   unsigned char *p;
   int len;

   if (!s || !*s) return s;
   ret = p = tmpstr(64);
   if (!ret) return s;

   while (*s) {
      if (*s < 32 && cmp[*s] && *cmp[*s]) {
         len = StrLen(cmp[*s]);
         StrNCopy(p, cmp[*s], len);
         p += len;
         s++;
      } else {
         *p++ = *s++;
      }
   }
   *p = 0;

   return ret;
}

#ifdef INIT_DATABASE
char * compress_string (unsigned char *s)
{
   char *ret;
   unsigned char *p;
   int i, len, replace;

   if (!s || !*s) return s;
   ret = p = tmpstr(64);
   if (!ret) return s;

   while (*s) {
      replace = 0;
      for (i = 1; i < 32 && !replace; i++) {
         if (cmp[i] && *cmp[i]) {
	    len = StrLen(cmp[i]);
	    if (! StrNCompare(cmp[i], s, len)) {
	       *p++ = i;
	       s += len;
	       replace = 1;
	    }
	 }
      }
      if (!replace) *p++ = *s++;
   }

   *p = 0;
   return ret;
}
#endif

/*
 * return a newly allocated block for string <s>. Cut off string <s> for
 * a maximum length of <len> chars. Always ensure a null byte follows.
 */
char *save_string(char *s, int len)
{
   int l;
   char *ret, *p;

   if (!s || !*s) return s;
   l = StrLen(s);
   if (len > 0  && len < l) l = len;
   p = ret = MemPtrNew (l + 1);
   if (!ret) return NULL;		// Oops, no memory ???
   while ((p - ret) < l) *p++ = *s++;	// strncpy()
   *p = 0;
   return ret;
}
