/*
    PubTrans - Scheduling public transportation on PalmPilot handheld

    Copyright (C) 1999 Andreas Kretzer

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    e-mail: andi@kretzer-berlin.de
*/

#include <Common.h>
#include <System/SysAll.h>
#include <UI/UIAll.h>
#include "pubtrans.h"

extern struct global_db global_db;
extern UInt times[];
wlist *select_list;
struct line *time_table;
char station_name[60] = { 0 };
ULong station;
UInt day;

static void time_draw (VoidPtr table, Word row, Word column,
		       RectanglePtr bounds)
{
   DateTimeType dt;
   TablePtr tp;
   TableItemPtr tip;
   short font;
   Int w, len;
   Boolean trunc;
   UInt time, curtime;
   char *s, *p, buf[80];

   tp = (TablePtr) table;
   tip = tp->items;

   WinEraseRectangle(bounds, 0);	// clear Cell
   TimSecondsToDateTime(TimGetSeconds(), &dt);
   curtime = (dt.hour * 60 + dt.minute) * 2;

   if (row == 0) {
      s = tip[tp->numRows * row + column].ptr;
      buf[0] = 0;
      p = 0;
      if (s) {
	 StrPrintF(buf, "%s", s);
         p = StrChr(buf, ' ');
         if (p) *p++ = 0;
      }
      font = FntSetFont(1);
      w = bounds->extent.x;
      len = StrLen(buf);
      trunc = false;
      FntCharsInWidth (buf, &w, &len, &trunc);
      buf[len] = 0;
      WinDrawChars (buf, StrLen(buf), bounds->topLeft.x + 1, bounds->topLeft.y);
      FntSetFont(font);
      if (p) {
	 w = bounds->extent.x;
	 len = StrLen(p);
	 trunc = false;
	 FntCharsInWidth (p, &w, &len, &trunc);
	 p[len] = 0;
	 WinDrawChars (p, StrLen(p), bounds->topLeft.x + 1, bounds->topLeft.y + 10);
      }
   } else {
      time = TblGetItemInt(table, row, column);
      if (time != 0xffff) {
	 font = FntSetFont((time > curtime) ? 1 : 0);
	 StringPrintTime (buf, time);
	 WinDrawChars (buf, StrLen(buf), bounds->topLeft.x + 5,
	 				 bounds->topLeft.y);
	 FntSetFont(font);
      }
   }

   // underline row 0
   if (row == 0) WinDrawLine (bounds->topLeft.x,
      		bounds->topLeft.y + bounds->extent.y - 1,
      		bounds->topLeft.x + bounds->extent.x,
		bounds->topLeft.y + bounds->extent.y - 1);

   // draw right border on column 0
   if (column == 0) WinDrawLine (bounds->topLeft.x + bounds->extent.x,
		bounds->topLeft.y,
		bounds->topLeft.x + bounds->extent.x,
		bounds->topLeft.y + bounds->extent.y - 1);

}

static void init_table(TablePtr table, int col, UInt time)
{
   int row, numrows;

   numrows = TblGetNumberOfRows(table);
   TblSetRowHeight(table, 0, 21);
   for (row = 0; row < numrows; row++) {
      TblSetRowUsable(table, row, true);
      TblMarkRowInvalid(table, row);
      TblSetItemStyle(table, row, 0, customTableItem);
      TblSetItemStyle(table, row, 1, customTableItem);
      TblSetItemPtr(table, row, 0, NULL);
      TblSetItemPtr(table, row, 1, NULL);
      TblSetItemInt(table, row, 0, 0xffff);
      TblSetItemInt(table, row, 1, 0xffff);
   }

   TblSetCustomDrawProcedure(table, 0, time_draw);
   TblSetCustomDrawProcedure(table, 1, time_draw);
   TblSetColumnUsable(table, 0, true);
   TblSetColumnUsable(table, 1, true);
   table_setvals (time_table, time, col, table);
}

static int lookup_time_idx(UInt time)
{
   int idx = 0;

   while (idx < MAXTIMES && times[idx] <= time) idx++;
   if (idx > 0) idx--;
   return idx;
}

static void redraw_table (FormPtr form, TablePtr table, int col, UInt time)
{
   init_table (table, col, time);

   TblMarkTableInvalid(table);
   TblRedrawTable(table);
   FrmDrawForm(form);
}

/*
 * handle events in timetable screen
 */
static Boolean timeline (EventType *event)
{
   static int cur_col = 0;
   static short time_idx = 0;
   static UInt time;
   static short max_col, sc_max;

   DateTimeType dt;
   FormPtr form;
   TablePtr table;
   ScrollBarPtr scroll;
   ControlPtr day_button, stat;

   int handled = 0;
   Int w, len;
   Boolean trunc;
   UInt curtime;
   char title[30];
   short sc_size, dummy;

   form = FrmGetActiveForm();
   table = FrmGetObjectPtr(form, FrmGetObjectIndex(form, tableID_timetable));
   scroll = FrmGetObjectPtr(form, FrmGetObjectIndex(form, scrollID_station));
   sc_size = TblGetNumberOfRows(table) - 1;
   stat = FrmGetObjectPtr(form, FrmGetObjectIndex(form, labelID_station));

   TimSecondsToDateTime(TimGetSeconds(), &dt);
   StrPrintF(title, "%02d:%02d", dt.hour, dt.minute);
   FrmSetTitle (form, title);	// always update current time
   curtime = (dt.hour * 60 + dt.minute) * 2;

   switch (event->eType) {
      case frmOpenEvent:
	 cur_col = 0;
	 if (time_table) {
	    free_lines (time_table);
	    time_table = NULL;
	 }
	 time_table = get_timetables(station, &max_col);
	 sc_max = calc_ruler (time_table, cur_col, station, day);

	 time_idx = lookup_time_idx(curtime);
	 time = times[time_idx];
	 init_table (table, cur_col, time);

	 SclSetScrollBar(scroll, time_idx, 0, sc_max, sc_size);

	 day_button = FrmGetObjectPtr(form, FrmGetObjectIndex(form, buttonID_su + day));
	 CtlSetValue (day_button, 1);

	 w = 100;
	 len = StrLen(station_name);
	 trunc = false;
	 FntCharsInWidth (station_name, &w, &len, &trunc);
	 station_name[len] = 0;
	 CtlSetLabel (stat, station_name);

	 FrmDrawForm(form);
	 handled = 1;
	 break;

      case frmCloseEvent:
	 if (time_table) {
	    free_lines (time_table);
	    time_table = NULL;
	 }
         break;

      case tblSelectEvent:
         TblUnhighlightSelection(table);
         handled = 1;
         break;

      case ctlSelectEvent:
         switch (event->data.ctlSelect.controlID) {
            case buttonID_back:
	       handled = 1;
	       FrmGotoForm(formID_stations);
	       break;
            case buttonID_su:
            case buttonID_mo:
            case buttonID_tu:
            case buttonID_we:
            case buttonID_th:
            case buttonID_fr:
            case buttonID_sa:
               day = event->data.ctlSelect.controlID - buttonID_su;
	       day_button = FrmGetObjectPtr(form, FrmGetObjectIndex(form, buttonID_su + day));
	       CtlSetValue (day_button, 0);
               free_timetables(time_table);
	       sc_max = calc_ruler (time_table, cur_col, station, day);

	       time_idx = lookup_time_idx(curtime);
	       time = times[time_idx];
	       redraw_table(form, table, cur_col, time);

	       SclSetScrollBar(scroll, time_idx, 0, sc_max, sc_size);
	       CtlSetValue (day_button, 1);
	       handled = 1;
	       break;

	    case buttonID_left:
	       cur_col -= 4;
	       /* FALL THRU */
	    case buttonID_right:
	       cur_col += 2;
	       if (cur_col > max_col - 2) cur_col = max_col - 2;
	       if (cur_col < 0) cur_col = 0;

	       sc_max = calc_ruler (time_table, cur_col, station, day);
	       time_idx = lookup_time_idx(time);
	       time = times[time_idx];

	       redraw_table(form, table, cur_col, time);

	       SclSetScrollBar(scroll, time_idx, 0, sc_max, sc_size);

	       handled = 1;
	       break;
         }
	 break;

      case sclRepeatEvent:
         SclGetScrollBar(scroll, &time_idx, &dummy, &dummy, &dummy);
	 time = times[time_idx];
	 redraw_table(form, table, cur_col, time);

         // handled = 1;
         break;

      case keyDownEvent:
         if (event->data.keyDown.chr > 0 && event->data.keyDown.chr < 31) {
	    switch (event->data.keyDown.chr) {
	       case 11:		/* KEY-Up */
		  time_idx -= sc_size;
		  /* FALL THRU */
	       case 12:		/* KEY-Down */
		  time_idx += sc_size / 2;
		  if (time_idx < 0) time_idx = 0;
		  if (time_idx > sc_max) time_idx = sc_max;
		  sc_max = calc_ruler (time_table, cur_col, station, day);

		  time = times[time_idx];
		  redraw_table(form, table, cur_col, time);
		  SclSetScrollBar(scroll, time_idx, 0, sc_max, sc_size);

	          handled = 1;
	          break;
	    }
	 }
   }

   return handled;
}

static Boolean station_select (EventType *event)
{
   FormPtr form;
   ListPtr list;
   FieldPtr search;
   int handled = 0;
   int i, len, rc;
   char tmp[32], tmp1[32], tmp2[32], tmp3[32];
   CharPtr s1, s2;

   form = FrmGetActiveForm();

   switch (event->eType) {
      case frmOpenEvent:
	 list = FrmGetObjectPtr(form, FrmGetObjectIndex(form, listID_stations));
	 LstEraseList(list);
	 select_list = get_stations(select_list);
	 LstSetListChoices (list, select_list->wtab, select_list->nwords);
	 LstSetTopItem(list, 0);
	 search = FrmGetObjectPtr(form, FrmGetObjectIndex(form, fieldID_search));
	 FrmDrawForm(form);
	 handled = 1;
	 break;

      case frmCloseEvent:
         delwlist(select_list);
         select_list = NULL;
         break;

      case frmUpdateEvent:
	 list = FrmGetObjectPtr(form, FrmGetObjectIndex(form, listID_stations));
	 FrmDrawForm(form);
	 LstDrawList(list);
	 handled = 1;
	 break;

      case keyDownEvent:
         if (event->data.keyDown.chr > 31) {
	    if (!select_list || select_list->nwords <= 0) break;
	    search = FrmGetObjectPtr(form, FrmGetObjectIndex(form, fieldID_search));
	    list = FrmGetObjectPtr(form, FrmGetObjectIndex(form, listID_stations));
            s1 = FldGetTextPtr(search);
            *tmp = 0;
            if (s1 && *s1) StrCopy (tmp, s1);
	    s1 = tmp + StrLen(tmp);
            *(s1++) = event->data.keyDown.chr; // copy the actual char to <tmp>
            *s1 = 0;
            s1 = tmp;
	    len = StrLen(s1);
	    if (len <= 0) break;
	    i = 0;
	    while (i < select_list->nwords) {
	       s2 = select_list->wtab[i];
	       rc = StrNCaselessCompare(s1, s2, len);
	       if (rc <= 0) {
		  if (rc < 0) {
		     i--;
		     handled = 1;
		     SndPlaySystemSound(sndWarning);
		  }
		  break;
	       }
	       i++;
	    }
	    if (i >= 0 && i < select_list->nwords) {
	       rc = i - 3;
	       if (rc < 0) rc = 0;
	       // LstSetSelection(list, -1);
	       // LstSetTopItem(list, rc);
	       // LstDrawList(list);
	       LstSetSelection(list, i);
            }
         } else if (event->data.keyDown.chr > 0) {
	    list = FrmGetObjectPtr(form, FrmGetObjectIndex(form, listID_stations));
	    switch (event->data.keyDown.chr) {
	       case 11:		/* KEY-Up */
	          LstScrollList(list, up, 11);
	          handled = 1;
	          break;
	       case 12:		/* KEY-Down */
	          LstScrollList(list, down, 11);
	          handled = 1;
	          break;
	    }
         } else {
	    StrPrintF(tmp1, "keyDownEvent");
            StrPrintF(tmp2, "character: %d", event->data.keyDown.chr);
            StrPrintF(tmp3, "button: %d", event->data.keyDown.keyCode);
            FrmCustomAlert(alertID_info,  tmp1, tmp2, tmp3);
	 }
         break;

      case lstSelectEvent:
	 list = FrmGetObjectPtr(form, FrmGetObjectIndex(form, listID_stations));
	 LstEraseList(list);
	 station = event->data.lstSelect.selection;
	 StrCopy (station_name, select_list->wtab[station]);
	 FrmGotoForm(formID_time);
         break;

      case ctlSelectEvent:
	 break;
      case nilEvent:
	 break;
   }
   return handled;
}

static void EventLoop (void)
{
   short err;
   int formID;
   long tout = -1;
   FormPtr form;
   EventType event;

   do {
      EvtGetEvent(&event, tout);
      if (SysHandleEvent (&event)) continue;
      if (MenuHandleEvent ((void *) 0, &event, &err)) continue;

      switch (event.eType) {
	 case frmLoadEvent:
	    formID = event.data.frmLoad.formID;
	    form = FrmInitForm(formID);
	    FrmSetActiveForm(form);
	    switch (formID) {
	       case formID_stations:
		  tout = -1;
		  FrmSetEventHandler(form, (FormEventHandlerPtr) station_select);
		  break;
	       case formID_time:
		  tout = 1000;
		  FrmSetEventHandler(form, (FormEventHandlerPtr) timeline);
		  break;
	    }
      }
      FrmDispatchEvent(&event);
   } while (event.eType != appStopEvent);
}

static int StartApplication (void)
{
   DateTimeType dt;

   /* init globals */
   select_list = NULL;
   time_table = NULL;
   station = 0;

   TimSecondsToDateTime (TimGetSeconds(), &dt);
   day = dt.weekDay;

   if (OpenDatabase()) {
      CloseDatabase();
      return -1;
   }
   FrmGotoForm(formID_stations);
   return 0;
}

static void StopApplication (void)
{
   CloseDatabase();
}

DWord PilotMain (Word cmd, Ptr cmdPBP, Word launchFlags)
{
   Word error;

   switch (cmd) {
      case sysAppLaunchCmdNormalLaunch:
	 error = StartApplication();
	 if (error) return error;
	 EventLoop();
	 StopApplication();
	 break;
   }
   return 0;
}
