/*
    PubTrans - Scheduling public transportation on PalmPilot handheld

    Copyright (C) 1999 Andreas Kretzer

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    e-mail: andi@kretzer-berlin.de
*/

#include <Common.h>
#include <System/SysAll.h>
#include <UI/UIAll.h>
#include "wlists.h"
#include "pt_proto.h"

/*
 * WLISTS for the PalmPilot
 */

long _wlistcount = 0;

wlist * genwlist()
{
   wlist *ret;

   ret = (wlist *) MemPtrNew(sizeof(wlist));
   if (!ret) return NULL;
   _wlistcount++;

   MemSet(ret, sizeof(wlist), 0);
   return ret;
}

void incrwlist(wlist * w)
{
   char **tmp;
   long *ltmp;
   Err rc;
   int n;

   if (w->ntab < 32) {
      n = w->ntab * 2;
      if (n < 4) n = 4;
   } else {
      n = w->ntab + 32;
   }

   if (w->wtab) {
      rc = MemPtrResize(w->wtab, n * sizeof(char *));
      if (rc) {
	 tmp = MemPtrNew(n * sizeof(char *));
	 if (!tmp) return;
	 MemMove(tmp, w->wtab, w->ntab * sizeof(char *));
	 w->wtab = tmp;
      }
   } else {
      w->wtab = MemPtrNew(n * sizeof(char *));
   }
   if (w->ltab) {
      rc = MemPtrResize(w->ltab, n * sizeof(long));
      if (rc) {
	 ltmp = MemPtrNew(n * sizeof(long));
	 if (!ltmp) return;
	 MemMove(ltmp, w->ltab, w->ntab * sizeof(long));
	 w->ltab = ltmp;
      }
   } else {
      w->ltab = MemPtrNew(n * sizeof(long));
   }
   if (w->wtab && w->ltab) {
      MemSet(&(w->wtab[w->ntab]), (n - w->ntab) * sizeof(char *), 0);
      MemSet(&(w->ltab[w->ntab]), (n - w->ntab) * sizeof(long), 0);
      w->ntab = n;
   }
}

wlist * addwlist(w, s, l) wlist * w; char * s; long l;
{
   if (!s) return w;
   if (w->nwords >= w->ntab) incrwlist(w);
   if (w->nwords < w->ntab) {
      w->wtab[w->nwords] = save_string(s, 0);
      w->ltab[w->nwords++] = l;
   }
   return w;
}

void delwlist(w) wlist * w;
{
   int i;

   if (!w) return;
   if (w->wtab) {
      for (i = 0; i < w->nwords; i++) {
	 if (w->wtab[i]) MemPtrFree(w->wtab[i]);
      }
      MemPtrFree(w->wtab);
   }
   if (w->ltab) MemPtrFree(w->ltab);
   MemPtrFree(w);
   _wlistcount--;
}
